@@ -26,8 +26,9 @@
 #include "request.h"
 #include "resources.h"
 
-#include <gnunet/gnunet_chat_lib.h>
+#include <glib-2.0/glib.h>
 #include <gnunet/gnunet_common.h>
+#include <gnunet/gnunet_chat_lib.h>
 #include <gstreamer-1.0/gst/gst.h>
 #include <gtk-3.0/gtk/gtk.h>
 #include <libhandy-1/handy.h>
@@ -66,6 +67,7 @@ _application_accounts(gpointer user_data)
   MESSENGER_Application *app = (MESSENGER_Application*) user_data;
 
   app->init = 0;
+  app->ui.state = MESSENGER_STATE_ACCOUNTS;
 
   ui_accounts_dialog_init(app, &(app->ui.accounts));
   ui_accounts_dialog_refresh(app, &(app->ui.accounts));
@@ -97,6 +99,8 @@ _application_init(MESSENGER_Application *app)
 {
   g_assert(app);
 
+  schedule_load_glib(&(app->ui.schedule));
+
   ui_messenger_init(app, &(app->ui.messenger));
 
 #ifndef MESSENGER_APPLICATION_NO_PORTAL
@@ -106,6 +110,8 @@ _application_init(MESSENGER_Application *app)
 
   if (app->chat.identity)
   {
+    app->ui.state = MESSENGER_STATE_MAIN_WINDOW;
+
     GNUNET_CHAT_iterate_accounts(
       app->chat.messenger.handle,
       _application_select_account,
@@ -179,92 +185,6 @@ _application_open(GApplication* application,
   g_application_release(application);
 }
 
-static void
-on_core_done(void *data,
-             UNUSED uint32_t id,
-             int seq)
-{
-  g_assert(data);
-
-	MESSENGER_Application *app = (MESSENGER_Application*) data;
-
-	if ((seq == app->pw.pending) && (app->pw.main_loop))
-		pw_main_loop_quit(app->pw.main_loop);
-}
-
-static void
-on_core_error(void *data,
-              UNUSED uint32_t id,
-              UNUSED int seq,
-              int res,
-              const char *message)
-{
-  g_assert((data) && (message));
-
-	MESSENGER_Application *app = (MESSENGER_Application*) data;
-
-  g_printerr("ERROR: %s\n", message);
-
-	if ((id == PW_ID_CORE) && (res == -EPIPE) && (app->pw.main_loop))
-		pw_main_loop_quit(app->pw.main_loop);
-}
-
-static const struct pw_core_events remote_core_events = {
-	PW_VERSION_CORE_EVENTS,
-	.done = on_core_done,
-	.error = on_core_error,
-};
-
-static void
-registry_event_global(void *data,
-                      uint32_t id,
-                      uint32_t permissions,
-                      const char *type,
-                      uint32_t version,
-                      const struct spa_dict *props)
-{
-  g_assert(data);
-
-  MESSENGER_Application *app = (MESSENGER_Application*) data;
-
-	if (!props)
-    return;
-
-  struct pw_properties *properties = pw_properties_new_dict(props);
-  if (!properties)
-    return;
-
-  size_t size = pw_map_get_size(&(app->pw.globals));
-	while (id > size)
-		pw_map_insert_at(&(app->pw.globals), size++, NULL);
-
-	pw_map_insert_at(&(app->pw.globals), id, properties);
-
-  app->pw.pending = app->pw.core? pw_core_sync(app->pw.core, 0, 0) : 0;
-}
-
-static void
-registry_event_global_remove(void *data,
-                             uint32_t id)
-{
-  g_assert(data);
-
-  MESSENGER_Application *app = (MESSENGER_Application*) data;
-
-  struct pw_properties *properties = pw_map_lookup(&(app->pw.globals), id);
-  if (!properties)
-    return;
-
-  pw_map_insert_at(&(app->pw.globals), id, NULL);
-	pw_properties_free(properties);
-}
-
-static const struct pw_registry_events registry_events = {
-	PW_VERSION_REGISTRY_EVENTS,
-	.global = registry_event_global,
-	.global_remove = registry_event_global_remove,
-};
-
 void
 application_init(MESSENGER_Application *app,
                  int argc,
@@ -298,6 +218,7 @@ application_init(MESSENGER_Application *app,
 #ifndef MESSENGER_APPLICATION_NO_PORTAL
   GError *error = NULL;
   app->portal = xdp_portal_initable_new(&error);
+  app->session = NULL;
 
   if (error)
   {
@@ -312,11 +233,11 @@ application_init(MESSENGER_Application *app,
 
   _load_ui_stylesheets(app);
 
+  schedule_init(&(app->chat.schedule));
+  schedule_init(&(app->ui.schedule));
+
   app->chat.status = EXIT_FAILURE;
   app->chat.tid = 0;
-  pipe(app->chat.pipe);
-
-  pthread_mutex_init(&(app->chat.mutex), NULL);
 
   app->quarks.widget = g_quark_from_string("messenger_widget");
   app->quarks.data = g_quark_from_string("messenger_data");
@@ -441,11 +362,6 @@ application_run(MESSENGER_Application *app)
   // Wait for other thread to stop properly
   pthread_join(app->chat.tid, NULL);
 
-  close(app->chat.pipe[0]);
-  close(app->chat.pipe[1]);
-
-  pthread_mutex_destroy(&(app->chat.mutex));
-
   GList *list;
   
   // Get rid of open requests
@@ -486,98 +402,61 @@ application_run(MESSENGER_Application *app)
   g_object_unref(app->application);
 }
 
-void
-application_pw_core_init(MESSENGER_Application *app)
-{
-  g_assert(app);
-
-  application_pw_core_cleanup(app);
-
-  if (app->pw.context)
-  {
-#ifndef MESSENGER_APPLICATION_NO_PORTAL
-    if (app->portal)
-      app->pw.core = pw_context_connect_fd(
-        app->pw.context,
-        xdp_portal_open_pipewire_remote_for_camera(app->portal),
-        NULL,
-        0
-      );
-    else
-#endif
-    app->pw.core = pw_context_connect(app->pw.context, NULL, 0);
-  }
-  else
-    app->pw.core = NULL;
-
-  app->pw.registry = app->pw.core? 
-    pw_core_get_registry(app->pw.core, PW_VERSION_REGISTRY, 0) : NULL;
-
-  pw_map_init(&(app->pw.globals), 64, 16);
-
-  if (app->pw.core)
-    pw_core_add_listener(
-      app->pw.core,
-      &(app->pw.core_listener),
-      &remote_core_events,
-      app
-    );
-
-  if (app->pw.registry)
-    pw_registry_add_listener(
-      app->pw.registry,
-      &(app->pw.registry_listener),
-      &registry_events,
-      app
-    );
-}
-
-static int
-destroy_global(void *obj,
-               UNUSED void *data)
-{
-  struct pw_properties *properties = (struct pw_properties*) obj;
-
-  if (!properties)
-    return 0;
-
-	pw_properties_free(properties);
-	return 0;
-}
-
-void
-application_pw_core_cleanup(MESSENGER_Application *app)
-{
-  g_assert(app);
-
-  if (app->pw.registry)
-    pw_proxy_destroy((struct pw_proxy*) app->pw.registry);
-
-  app->pw.registry = NULL;
-
-  pw_map_for_each(&(app->pw.globals), destroy_global, NULL);
-  pw_map_clear(&(app->pw.globals));
-
-  if (app->pw.core)
-    pw_core_disconnect(app->pw.core);
-
-  app->pw.core = NULL;
-}
-
 void
 application_pw_main_loop_run(MESSENGER_Application *app)
 {
   g_assert(app);
 
-  if ((! app->pw.core) || (! app->pw.main_loop))
+  if (!(app->pw.main_loop))
     return;
 
-  app->pw.pending = app->pw.core? pw_core_sync(app->pw.core, 0, 0) : 0;
-
-  if ((app->pw.main_loop) && (app->pw.core))
-    pw_main_loop_run(app->pw.main_loop);
+  pw_main_loop_run(app->pw.main_loop);
 }
 
+#ifndef MESSENGER_APPLICATION_NO_PORTAL
+void
+application_set_active_session(MESSENGER_Application *app,
+                               XdpSession *session)
+{
+  g_assert(app);
+
+  if (app->session == session)
+    return;
+
+  if (app->session)
+  {
+    const XdpSessionState state = xdp_session_get_session_state(
+      app->session
+    );
+
+    if (XDP_SESSION_ACTIVE == state)
+      xdp_session_close(app->session);
+
+    g_object_unref(app->session);
+  }
+
+  app->session = session;
+}
+
+int
+application_get_active_session_remote(MESSENGER_Application *app)
+{
+  g_assert(app);
+
+  if (!(app->session))
+    return -1;
+
+  const XdpSessionState state = xdp_session_get_session_state(
+    app->session
+  );
+
+  if (XDP_SESSION_ACTIVE != state)
+    return -1;
+  
+  return xdp_session_open_pipewire_remote(app->session);
+}
+#endif
+
 static void
 _request_background_callback(MESSENGER_Application *app,
                              gboolean success,
@@ -594,6 +473,9 @@ void
 application_show_window(MESSENGER_Application *app)
 {
   g_assert(app);
+  
+  if (MESSENGER_STATE_MAIN_WINDOW != app->ui.state)
+    return;
 
   gtk_widget_show(GTK_WIDGET(app->ui.messenger.main_window));
 
@@ -627,13 +509,10 @@ _application_event_call(gpointer user_data)
 
   call = (MESSENGER_ApplicationEventCall*) user_data;
 
-  // Locking the mutex for synchronization
-  pthread_mutex_lock(&(call->app->chat.mutex));
   call->event(call->app);
-  pthread_mutex_unlock(&(call->app->chat.mutex));
 
   GNUNET_free(call);
-  return FALSE;
+  return TRUE;
 }
 
 void
@@ -651,7 +530,11 @@ application_call_event(MESSENGER_Application *app,
   call->app = app;
   call->event = event;
 
-  util_immediate_add(G_SOURCE_FUNC(_application_event_call), call);
+  schedule_sync_run(
+    &(app->ui.schedule),
+    G_SOURCE_FUNC(_application_event_call),
+    call
+  );
 }
 
 static gboolean
@@ -664,12 +547,9 @@ _application_sync_event_call(gpointer user_data)
   call = (MESSENGER_ApplicationEventCall*) user_data;
 
   call->event(call->app);
-  util_scheduler_cleanup();
-
-  pthread_mutex_unlock(&(call->app->chat.mutex));
 
   GNUNET_free(call);
-  return FALSE;
+  return TRUE;
 }
 
 void
@@ -687,11 +567,11 @@ application_call_sync_event(MESSENGER_Application *app,
   call->app = app;
   call->event = event;
 
-  util_scheduler_cleanup();
-  util_immediate_add(G_SOURCE_FUNC(_application_sync_event_call), call);
-
-  // Locking the mutex for synchronization
-  pthread_mutex_lock(&(app->chat.mutex));
+  schedule_sync_run(
+    &(app->ui.schedule),
+    G_SOURCE_FUNC(_application_sync_event_call),
+    call
+  );
 }
 
 typedef struct MESSENGER_ApplicationMessageEventCall
@@ -712,13 +592,10 @@ _application_message_event_call(gpointer user_data)
 
   call = (MESSENGER_ApplicationMessageEventCall*) user_data;
 
-  // Locking the mutex for synchronization
-  pthread_mutex_lock(&(call->app->chat.mutex));
   call->event(call->app, call->context, call->message);
-  pthread_mutex_unlock(&(call->app->chat.mutex));
 
   GNUNET_free(call);
-  return FALSE;
+  return TRUE;
 }
 
 void
@@ -741,7 +618,52 @@ application_call_message_event(MESSENGER_Application *app,
   call->context = context;
   call->message = message;
 
-  util_immediate_add(G_SOURCE_FUNC(_application_message_event_call), call);
+  schedule_sync_run(
+    &(app->ui.schedule),
+    G_SOURCE_FUNC(_application_message_event_call),
+    call
+  );
+}
+
+void
+application_chat_lock(MESSENGER_Application *app)
+{
+  g_assert(app);
+
+  if (app->ui.schedule.function)
+  {
+    g_assert(!(app->chat.schedule.locked));
+    return;
+  }
+  
+  schedule_sync_lock(&(app->chat.schedule));
+}
+
+void
+application_chat_unlock(MESSENGER_Application *app)
+{
+  g_assert(app);
+
+  if (app->ui.schedule.function)
+  {
+    g_assert(!(app->chat.schedule.locked));
+    return;
+  }
+
+  schedule_sync_unlock(&(app->chat.schedule));
+}
+
+static gboolean
+_application_stop_chat(gpointer user_data)
+{
+  MESSENGER_Application *app = user_data;
+
+  GNUNET_CHAT_disconnect(app->chat.messenger.handle);
+  GNUNET_CHAT_stop(app->chat.messenger.handle);
+  app->chat.messenger.handle = NULL;
+
+  GNUNET_SCHEDULER_shutdown();
+  return FALSE;
 }
 
 void
@@ -750,11 +672,19 @@ application_exit(MESSENGER_Application *app,
 {
   g_assert(app);
 
-  // Forward a signal to the other thread causing it to shutdown the
-  // GNUnet handles of the application.
-  write(app->chat.pipe[1], &signal, sizeof(signal));
+  schedule_sync_run(
+    &(app->chat.schedule),
+    G_SOURCE_FUNC(_application_stop_chat),
+    app
+  );
 
-  application_pw_core_cleanup(app);
+  schedule_cleanup(&(app->chat.schedule));
+  schedule_cleanup(&(app->ui.schedule));
+
+  util_scheduler_cleanup();
+
+  media_pw_cleanup(&(app->media.camera));
+  media_pw_cleanup(&(app->media.screen));
 
   if (app->pw.context)
     pw_context_destroy(app->pw.context);
@@ -766,6 +696,8 @@ application_exit(MESSENGER_Application *app,
   }
 
 #ifndef MESSENGER_APPLICATION_NO_PORTAL
+  application_set_active_session(app, NULL);
+
   if (app->portal)
     g_object_unref(app->portal);
 
