@@ -30,6 +30,9 @@
 typedef struct MESSENGER_ContactInfo
 {
   void *last_message;
+  GFile *icon_file;
+  GIcon *icon;
+  guint task;
 
   GList *name_labels;
   GList *name_avatars;
@@ -42,8 +45,9 @@ typedef struct MESSENGER_ContactInfo
  * once.
  *
  * @param contact Chat contact
+ * @return #GNUNET_YES on info creation, otherwise #GNUNET_NO
  */
-void
+enum GNUNET_GenericReturnValue
 contact_create_info(struct GNUNET_CHAT_Contact *contact);
 
 /**
@@ -146,8 +150,20 @@ contact_remove_visible_widget_to_info(const struct GNUNET_CHAT_Contact *contact,
  * contact depending on the current state.
  *
  * @param contact Chat contact
+ * @param attributes Flag to check for attributes changes
  */
 void
 contact_update_info(const struct GNUNET_CHAT_Contact *contact);
 
+/**
+ * Updates the connected UI elements for a given
+ * contact based on current attributes changes.
+ *
+ * @param contact Chat contact
+ * @param app Messenger application
+ */
+void
+contact_update_attributes(struct GNUNET_CHAT_Contact *contact,
+                          MESSENGER_Application *app);
+
 #endif /* CONTACT_H_ */
