@@ -25,8 +25,8 @@
 #include "chat.h"
 
 #include <gdk/gdkkeysyms.h>
-#include <gnunet/gnunet_chat_lib.h>
 #include <gnunet/gnunet_common.h>
+#include <gnunet/gnunet_chat_lib.h>
 #include <gnunet/gnunet_time_lib.h>
 #include <stdlib.h>
 
@@ -44,6 +44,56 @@
 #include "../file.h"
 #include "../ui.h"
 
+static void
+handle_chat_details_folded(GObject* object,
+                           GParamSpec* pspec,
+                           gpointer user_data)
+{
+  g_assert((object) && (pspec) && (user_data));
+
+  HdyFlap* flap = HDY_FLAP(object);
+  UI_CHAT_Handle *handle = (UI_CHAT_Handle*) user_data;
+  UI_MESSENGER_Handle *messenger = &(handle->app->ui.messenger);
+
+  const gboolean revealed = hdy_flap_get_reveal_flap(flap);
+
+  hdy_leaflet_set_can_swipe_back(messenger->leaflet_title, !revealed);
+  hdy_leaflet_set_can_swipe_back(messenger->leaflet_chat, !revealed);
+
+  if (handle->title)
+  {
+    gtk_widget_set_sensitive(
+      GTK_WIDGET(handle->title->back_button),
+      !revealed
+    );
+
+    gtk_widget_set_sensitive(
+      GTK_WIDGET(handle->title->chat_search_button),
+      !revealed
+    );
+  }
+
+  GValue value = G_VALUE_INIT;
+  g_value_init(&value, G_TYPE_BOOLEAN);
+  g_value_set_boolean(&value, !revealed);
+
+  gtk_container_child_set_property(
+    GTK_CONTAINER(messenger->leaflet_title),
+    GTK_WIDGET(messenger->nav_bar),
+    "navigatable",
+    &value
+  );
+
+  gtk_container_child_set_property(
+    GTK_CONTAINER(messenger->leaflet_chat),
+    messenger->nav_box,
+    "navigatable",
+    &value
+  );
+
+  g_value_unset(&value);
+}
+
 static gboolean
 _flap_chat_details_reveal_switch(gpointer user_data)
 {
@@ -52,11 +102,9 @@ _flap_chat_details_reveal_switch(gpointer user_data)
   UI_CHAT_Handle *handle = (UI_CHAT_Handle*) user_data;
   HdyFlap* flap = handle->flap_chat_details;
 
-  if (TRUE == hdy_flap_get_reveal_flap(flap)) {
-    hdy_flap_set_reveal_flap(flap, FALSE);
-  } else {
-    hdy_flap_set_reveal_flap(flap, TRUE);
-  }
+  gboolean revealed = hdy_flap_get_reveal_flap(flap);
+
+  hdy_flap_set_reveal_flap(flap, !revealed);
 
   gtk_widget_set_sensitive(GTK_WIDGET(handle->messages_listbox), TRUE);
   return FALSE;
@@ -117,9 +165,13 @@ handle_chat_contacts_listbox_row_activated(GtkListBox *listbox,
 
   hdy_flap_set_reveal_flap(handle->flap_chat_details, FALSE);
 
+  application_chat_lock(app);
+
   ui_contact_info_dialog_init(app, &(app->ui.contact_info));
   ui_contact_info_dialog_update(&(app->ui.contact_info), contact, FALSE);
 
+  application_chat_unlock(app);
+
   gtk_widget_show(GTK_WIDGET(app->ui.contact_info.dialog));
 }
 
@@ -166,12 +218,33 @@ handle_reveal_identity_button_click(GtkButton *button,
 
   hdy_flap_set_reveal_flap(handle->flap_chat_details, FALSE);
 
+  application_chat_lock(app);
+
   ui_contact_info_dialog_init(app, &(app->ui.contact_info));
   ui_contact_info_dialog_update(&(app->ui.contact_info), contact, TRUE);
 
+  application_chat_unlock(app);
+
   gtk_widget_show(GTK_WIDGET(app->ui.contact_info.dialog));
 }
 
+static void
+handle_discourse_button_click(GtkButton *button,
+				                      gpointer user_data)
+{
+  g_assert((button) && (user_data));
+
+  UI_CHAT_Handle *handle = (UI_CHAT_Handle*) user_data;
+  MESSENGER_Application *app = handle->app;
+
+  hdy_flap_set_reveal_flap(handle->flap_chat_details, FALSE);
+
+  ui_discourse_window_init(app, &(app->ui.discourse));
+  ui_discourse_window_update(&(app->ui.discourse), handle->context);
+
+  gtk_widget_show(GTK_WIDGET(app->ui.discourse.window));
+}
+
 static void
 handle_block_button_click(UNUSED GtkButton *button,
                           gpointer user_data)
@@ -187,7 +260,9 @@ handle_block_button_click(UNUSED GtkButton *button,
   if (!contact)
     return;
 
+  application_chat_lock(handle->app);
   GNUNET_CHAT_contact_set_blocked(contact, GNUNET_YES);
+  application_chat_unlock(handle->app);
 
   gtk_stack_set_visible_child(handle->block_stack, GTK_WIDGET(handle->unblock_button));
 }
@@ -207,7 +282,9 @@ handle_unblock_button_click(UNUSED GtkButton *button,
   if (!contact)
     return;
 
+  application_chat_lock(handle->app);
   GNUNET_CHAT_contact_set_blocked(contact, GNUNET_NO);
+  application_chat_unlock(handle->app);
 
   gtk_stack_set_visible_child(handle->block_stack, GTK_WIDGET(handle->block_button));
 }
@@ -223,6 +300,8 @@ handle_leave_chat_button_click(UNUSED GtkButton *button,
   if ((!handle) || (!(handle->context)))
     return;
 
+  application_chat_lock(handle->app);
+
   struct GNUNET_CHAT_Contact *contact = GNUNET_CHAT_context_get_contact(
     handle->context
   );
@@ -236,6 +315,8 @@ handle_leave_chat_button_click(UNUSED GtkButton *button,
   else if (group)
     GNUNET_CHAT_group_leave(group);
 
+  application_chat_unlock(handle->app);
+
   UI_CHAT_ENTRY_Handle *entry = GNUNET_CHAT_context_get_user_pointer(
     handle->context
   );
@@ -357,12 +438,16 @@ handle_chat_messages_filter(GtkListBoxRow *row,
     filterTags.filter = &(filter[1]);
     filterTags.matching = FALSE;
 
+    application_chat_lock(app);
+
     GNUNET_CHAT_message_iterate_tags(
       message->msg,
       _iterate_message_tags,
       &filterTags
     );
 
+    application_chat_unlock(app);
+
     result |= filterTags.matching;
   }
 
@@ -524,7 +609,11 @@ _send_text_from_view(MESSENGER_Application *app,
   }
 
   if (handle->context)
+  {
+    application_chat_lock(app);
     GNUNET_CHAT_context_send_text(handle->context, text);
+    application_chat_unlock(app);
+  }
 
   g_free(text);
   gtk_text_buffer_delete(buffer, &start, &end);
@@ -599,6 +688,8 @@ handle_send_record_button_click(GtkButton *button,
     ui_label_set_text(file_load->file_label, handle->recording_filename);
     gtk_progress_bar_set_fraction(file_load->load_progress_bar, 0.0);
 
+    application_chat_lock(app);
+
     struct GNUNET_CHAT_File *file = GNUNET_CHAT_context_send_file(
       handle->context,
       handle->recording_filename,
@@ -615,6 +706,8 @@ handle_send_record_button_click(GtkButton *button,
     else if (file_load)
       ui_file_load_entry_delete(file_load);
 
+    application_chat_unlock(app);
+
     _drop_any_recording(handle);
     return;
   }
@@ -1176,6 +1269,13 @@ ui_chat_new(MESSENGER_Application *app,
     gtk_builder_get_object(handle->builder, "flap_chat_details")
   );
 
+  g_signal_connect(
+    handle->flap_chat_details,
+    "notify::reveal-flap",
+    G_CALLBACK(handle_chat_details_folded),
+    handle
+  );
+
   handle->chat_search_bar = HDY_SEARCH_BAR(
     gtk_builder_get_object(handle->builder, "chat_search_bar")
   );
@@ -1226,6 +1326,17 @@ ui_chat_new(MESSENGER_Application *app,
     handle
   );
 
+  handle->discourse_button = GTK_BUTTON(
+    gtk_builder_get_object(handle->builder, "discourse_button")
+  );
+
+  g_signal_connect(
+    handle->discourse_button,
+    "clicked",
+    G_CALLBACK(handle_discourse_button_click),
+    handle
+  );
+
   handle->block_stack = GTK_STACK(
     gtk_builder_get_object(handle->builder, "block_stack")
   );
@@ -1451,6 +1562,12 @@ ui_chat_new(MESSENGER_Application *app,
     handle->send_text_view
   );
 
+  g_object_set_qdata(
+    G_OBJECT(handle->send_text_view),
+    app->quarks.data,
+    context
+  );
+
   g_object_set_qdata(
     G_OBJECT(handle->send_text_view),
     app->quarks.ui,
