@@ -25,6 +25,7 @@
 #include "contact_entry.h"
 
 #include "../application.h"
+#include "../contact.h"
 #include "../ui.h"
 
 UI_CONTACT_ENTRY_Handle*
@@ -34,7 +35,9 @@ ui_contact_entry_new(MESSENGER_Application *app)
 
   UI_CONTACT_ENTRY_Handle* handle = g_malloc(sizeof(UI_CONTACT_ENTRY_Handle));
 
-  handle->builder = gtk_builder_new_from_resource(
+  handle->contact = NULL;
+
+  handle->builder = ui_builder_from_resource(
     application_get_resource_path(app, "ui/contact_entry.ui")
   );
 
@@ -61,14 +64,25 @@ void
 ui_contact_entry_set_contact(UI_CONTACT_ENTRY_Handle* handle,
                              const struct GNUNET_CHAT_Contact *contact)
 {
-  g_assert((handle) && (contact));
+  g_assert(handle);
 
-  const char *name = GNUNET_CHAT_contact_get_name(contact);
-  const char *key = GNUNET_CHAT_contact_get_key(contact);
+  if (handle->contact)
+  {
+    contact_remove_name_avatar_from_info(handle->contact, handle->entry_avatar);
+    contact_remove_name_label_from_info(handle->contact, handle->title_label);
+  }
 
-  ui_avatar_set_text(handle->entry_avatar, name);
-  ui_label_set_text(handle->title_label, name);
-  ui_label_set_text(handle->subtitle_label, key);
+  if (contact)
+  {
+    const char *key = GNUNET_CHAT_contact_get_key(contact);
+
+    contact_add_name_avatar_to_info(contact, handle->entry_avatar);
+    contact_add_name_label_to_info(contact, handle->title_label);
+
+    ui_label_set_text(handle->subtitle_label, key);
+  }
+
+  handle->contact = contact;
 }
 
 void
@@ -76,6 +90,8 @@ ui_contact_entry_delete(UI_CONTACT_ENTRY_Handle *handle)
 {
   g_assert(handle);
 
+  ui_contact_entry_set_contact(handle, NULL);
+
   g_object_unref(handle->builder);
 
   g_free(handle);
