@@ -72,16 +72,25 @@ handle_file_button_click(GtkButton *button,
   if (!file)
     return;
 
+  application_chat_lock(app);
   uint64_t size = GNUNET_CHAT_file_get_size(file);
+  application_chat_unlock(app);
 
   if (size <= 0)
     return;
 
-  uint64_t local_size = GNUNET_CHAT_file_get_local_size(file);
+  application_chat_lock(app);
 
-  if (GNUNET_YES == GNUNET_CHAT_file_is_downloading(file))
+  uint64_t local_size = GNUNET_CHAT_file_get_local_size(file);
+  const gboolean downloading = (GNUNET_YES == GNUNET_CHAT_file_is_downloading(file));
+
+  application_chat_unlock(app);
+
+  if (downloading)
   {
+    application_chat_lock(app);
     GNUNET_CHAT_file_stop_download(file);
+    application_chat_unlock(app);
 
     gtk_image_set_from_icon_name(
       handle->file_status_image,
@@ -91,11 +100,13 @@ handle_file_button_click(GtkButton *button,
   }
   else if (local_size < size)
   {
+    application_chat_lock(app);
     GNUNET_CHAT_file_start_download(
       file,
       handle_downloading_file,
       app
     );
+    application_chat_unlock(app);
 
     gtk_image_set_from_icon_name(
     	handle->file_status_image,
@@ -105,7 +116,9 @@ handle_file_button_click(GtkButton *button,
   }
   else if (size > 0)
   {
+    application_chat_lock(app);
     const gchar *preview = GNUNET_CHAT_file_open_preview(file);
+    application_chat_unlock(app);
 
     if (!preview)
       return;
@@ -114,7 +127,11 @@ handle_file_button_click(GtkButton *button,
     g_string_append(uri, preview);
 
     if (!g_app_info_launch_default_for_uri(uri->str, NULL, NULL))
+    {
+      application_chat_lock(app);
       GNUNET_CHAT_file_close_preview(file);
+      application_chat_unlock(app);
+    }
 
     g_string_free(uri, TRUE);
   }
@@ -142,7 +159,9 @@ handle_media_button_click(GtkButton *button,
   if (!file)
     return;
 
+  application_chat_lock(app);
   const gchar *preview = GNUNET_CHAT_file_open_preview(file);
+  application_chat_unlock(app);
 
   if (!preview)
     return;
