@@ -41,6 +41,49 @@
 #include "../application.h"
 #include "../ui.h"
 
+static void
+handle_user_details_folded(GObject* object,
+                           GParamSpec* pspec,
+                           gpointer user_data)
+{
+  g_assert((object) && (pspec) && (user_data));
+
+  HdyFlap* flap = HDY_FLAP(object);
+  UI_MESSENGER_Handle *handle = (UI_MESSENGER_Handle*) user_data;
+
+  const gboolean revealed = hdy_flap_get_reveal_flap(flap);
+
+  gtk_widget_set_sensitive(
+    GTK_WIDGET(handle->chats_search_entry),
+    !revealed
+  );
+
+  gtk_widget_set_sensitive(
+    GTK_WIDGET(handle->chats_search_button),
+    !revealed
+  );
+
+  GValue value = G_VALUE_INIT;
+  g_value_init(&value, G_TYPE_BOOLEAN);
+  g_value_set_boolean(&value, !revealed);
+
+  gtk_container_child_set_property(
+    GTK_CONTAINER(handle->leaflet_title),
+    GTK_WIDGET(handle->main_bar),
+    "navigatable",
+    &value
+  );
+
+  gtk_container_child_set_property(
+    GTK_CONTAINER(handle->leaflet_chat),
+    handle->main_box,
+    "navigatable",
+    &value
+  );
+
+  g_value_unset(&value);
+}
+
 static void
 handle_profile_button_click(UNUSED GtkButton* button,
 				                             gpointer user_data)
@@ -169,17 +212,23 @@ handle_accounts_listbox_row_activated(UNUSED GtkListBox* listbox,
   if (!account)
     return;
 
+  application_chat_lock(app);
+  
   const struct GNUNET_CHAT_Account *current = GNUNET_CHAT_get_connected(
     app->chat.messenger.handle
   );
 
+  application_chat_unlock(app);
+
   if (account == current)
     return;
 
   _switch_details_revealer_visibility(&(app->ui.messenger), FALSE);
   hdy_flap_set_reveal_flap(HDY_FLAP(app->ui.messenger.flap_user_details), FALSE);
 
+  application_chat_lock(app);
   GNUNET_CHAT_connect(app->chat.messenger.handle, account);
+  application_chat_unlock(app);
 }
 
 static void
@@ -468,6 +517,21 @@ ui_messenger_init(MESSENGER_Application *app,
     gtk_builder_get_object(handle->builder, "flap_user_details")
   );
 
+  g_signal_connect(
+    handle->flap_user_details,
+    "notify::reveal-flap",
+    G_CALLBACK(handle_user_details_folded),
+    handle
+  );
+
+  handle->nav_box = GTK_WIDGET(
+    gtk_builder_get_object(handle->builder, "nav_box")
+  );
+
+  handle->main_box = GTK_WIDGET(
+    gtk_builder_get_object(handle->builder, "main_box")
+  );
+
   handle->nav_bar = HDY_HEADER_BAR(
     gtk_builder_get_object(handle->builder, "nav_bar")
   );
@@ -828,12 +892,16 @@ ui_messenger_refresh(MESSENGER_Application *app,
     handle->accounts_listbox
   );
 
+  application_chat_lock(app);
+
   GNUNET_CHAT_iterate_accounts(
     app->chat.messenger.handle,
     _messenger_iterate_accounts,
     app
   );
 
+  application_chat_unlock(app);
+
   if (gtk_list_box_get_selected_row(handle->accounts_listbox))
     return;
   
