@@ -37,6 +37,8 @@ _open_new_group(GtkEntry *entry,
 
   const gchar *name = gtk_entry_get_text(entry);
 
+  application_chat_lock(app);
+
   struct GNUNET_CHAT_Group *group = GNUNET_CHAT_group_create(
       app->chat.messenger.handle,
       NULL
@@ -45,6 +47,8 @@ _open_new_group(GtkEntry *entry,
   if ((name) && (strlen(name) > 0))
     GNUNET_CHAT_group_set_name(group, name);
 
+  application_chat_unlock(app);
+
   GList *selected = gtk_list_box_get_selected_rows(listbox);
 
   GList *item = selected;
@@ -58,7 +62,9 @@ _open_new_group(GtkEntry *entry,
 	      g_object_get_qdata(G_OBJECT(row), app->quarks.data)
       );
 
+      application_chat_lock(app);
       GNUNET_CHAT_group_invite_contact(group, contact);
+      application_chat_unlock(app);
     }
 
     item = item->next;
